/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTFrame_h__
#define __RTFrame_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTActorId_h__
#include <RTActorId.h>
#endif

#ifndef __RTController_h__
#include <RTController.h>
#endif

#ifndef __RTProtocol_h__
#include <RTProtocol.h>
#endif

struct RTActorClass;
class  RTActorRef;
class  RTController;
struct RTTypedValue;

struct Frame
{
    class Base : public RTProtocol
    {
    public:
	RTS_INLINE      Base( void );
	RTS_INLINE      ~Base( void );

	static const RTProtocolDescriptor rt_class;
	RTActorId       incarnate( RTActorRef & );

	RTActorId       incarnate( RTActorRef &, const RTActorClass & );

	RTActorId       incarnate( RTActorRef           &,
				   const void           * data,
				   const RTObject_class * type,
				   RTController         *,
				   int                    index );

	RTActorId       incarnate( RTActorRef           &,
				   const RTActorClass   &,
				   const void           * data,
				   const RTObject_class * type,
				   RTController         *,
				   int                    index );

	RTActorId       incarnate( RTActorRef         &,
				   const RTDataObject &,
				   RTController       *       = 0,
				   int                  index = -1 );

	RTActorId       incarnate( RTActorRef         &,
				   const RTActorClass &,
				   const RTDataObject &,
				   RTController       *       = 0,
				   int                  index = -1 );

	RTActorId       incarnate( RTActorRef         &,
				   const RTTypedValue & info,
				   RTController       *       = 0,
				   int                  index = -1 );

	RTActorId       incarnate( RTActorRef         &,
				   const RTActorClass &,
				   const RTTypedValue & info,
				   RTController       *       = 0,
				   int                  index = -1 );

	int             destroy( RTActorId  & );
	int             destroy( RTActorRef & );

	int             import( const RTActorId &,
				RTActorRef      &,
				int               index = -1 );
	int             import( RTActorRef & actor,
				RTActorRef & dest,
				int          index = -1 );
	int             deport( const RTActorId &, RTActorRef & );
	RTActorId    ** incarnationsOf( const RTActorRef & );
	RTActorId       incarnationAt ( const RTActorRef &, int index );
	RTActorId       me( void );

	const RTActorClass & myClass( void );
	const RTActorClass & classOf( const RTActorId & incarnation );

	int             classIsKindOf( const RTActorClass & subClass,
				       const RTActorClass & superClass );

	const char    * className( const RTActorClass & );

    private:
#if RTFRAME_CHECKING != RTFRAME_CHECK_NONE
	int             permissionCheck( RTController::Primitive,
					 RTActorRef              *,
					 int                       trace );
#endif

	// unavailable methods
	Base( const Base & );
	Base & operator=( const Base & );
    };
};

#if RTS_INLINES
#include <RTFrame.inl>
#endif

#endif // __RTFrame_h__
