#! perl -w

# Licensed Materials - Property of IBM                       
# Component IBM Rational ROSE_RT BUILD : TARGET_RTS                 
# Copyright IBM Corporation 1999, 2008. All Rights Reserved.      
# U.S. Government Users Restricted Rights -  Use, duplication or    
# disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.



# This script is necessary because Microsofts's compiler does not seem
# to have options that correspond to the usual -L and -l options.

sub fatal
{
    print STDERR "$_[0]\n";
    exit 1;
}

# Win9x command.com does not interact with perl according to win32 standard.
# We therefore need to call some functions in a non-standard way.
#
# This detects if the system is Win95 for that purpose.

sub mySystem
{
    my( $OS, $ROSERT_HOST );

    $OS          = '?' unless $OS          = $ENV{'OS'};
    $ROSERT_HOST = '?' unless $ROSERT_HOST = $ENV{'ROSERT_HOST'};

    if( $ROSERT_HOST eq 'win32' && $OS !~ m{^Windows_NT$}i )
    {
	return system( join( ' ', @_ ) );
    }
    else
    {
	return system( @_ );
    }
}

$#ARGV >= 0 || &fatal( "ld.pl: Usage: ${0} link options ... " );
$LIB_EXT= '.LIB';
$OBJ_EXT= '.OBJ';

@command  = ( shift @ARGV, '/nologo' );

@usrlib_dirs = ();
if( $lib = $ENV{'LIB'} )
{
    @lib_dirs = split( ';', $lib );
}
else
{
    @lib_dirs = ();
}
$status   = 0; # assume all is well

sub find_lib
{
    my( $lib ) = @_;
    my( $file );

    foreach $dir ( @usrlib_dirs, @lib_dirs )
    {
	$file = "$dir/$lib$LIB_EXT";
	if( -r "$file" )
	{
	    return $file;
	}
    }

    return undef();
}

sub quoted
{
    my( $arg ) = @_;

    return '"' .  $arg . '"' if( $arg =~ m{\s} );
    return        $arg;
}

sub backslashed
{
    my( $arg ) = @_;

    # The linker will treat things that begin with slash as switches
    # even if they are path-names.  So all file-paths for .OBJs or .LIBs should
    # start with a drive-letter or a backslash.
    # Other file-paths may also need to be treated similarly.

    if( ( $arg =~ m/$LIB_EXT$/i ) or ( $arg =~ m/$OBJ_EXT$/i ) )
    {
	$arg =~ s{^/}{\\};
    }

    return $arg;
}

$cmd_file = 'ld.cmd';
if( ! open( OUT, ">$cmd_file" ) )
{
    print STDERR "ld.pl: Can't create $cmd_file\n";
    exit 1;
}

foreach $arg ( @ARGV )
{
    if( $arg =~ m{^-l(.*)$} )
    {
	if( $lib = &find_lib( $1 ) )
	{
	    print OUT &quoted( &backslashed( $lib ) ), "\n";
	}
	else
	{
	    print STDERR "ld.pl: Can't find library for $arg\n";
	    $status = 1;
	}
    }
    elsif( $arg =~ m{^-L(.*)$} )
    {
	push @usrlib_dirs, $1;
    }
    elsif( $arg =~ m{^%(.*)$} )
    {
	$in  = $1;
	if( ! open( IN, "<$in" ) )
	{
	    print STDERR "ld.pl: Can't read '$in': $!\n";
	    $status = 1;
	}
	else
	{
	    while( <IN> )
	    {
		s{$}{$OBJ_EXT};
		print OUT;
	    }
	    close( IN );
	}
    }
    else
    {
	print OUT &quoted( &backslashed( $arg ) ), "\n";
    }
}

close( OUT );

push @command, &quoted( "\@$cmd_file" );

if( $status == 0 )
{
    $status = &mySystem( @command );
}

unlink( $cmd_file );

if( $status < 256 )
{
    exit $status;
}
else
{
    exit $status / 256;
}
