/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActor.h>
#include <RTActorClass.h>
#include <RTActorRef.h>
#include <RTComponentDescriptor.h>
#include <RTDebugger.h>

RTActor * RTActorRef::incarnate2( const RTActorClass * actorClass,
				  RTController       * rts,
				  int                  index )
{
    RTActor      * myOwner    = getOwner();
    RTController * controller = myOwner->context();

    controller->trace( RTController::FrameIncarnate, this, &index );

    if( _info->kind == RTComponentDescriptor::Imported )
    {
	controller->setError( RTController::imported,
			      RTController::FrameIncarnate, this, &index );
	return (RTActor *)0;
    }

    RTController::Error error = index < 0 ? RTController::refFull
					  : RTController::badIndex;

    if( reserve( index ) == 0 )
    {
	controller->setError( error, RTController::FrameIncarnate, this,
								   &index );
	return (RTActor *)0;
    }

    if( rts == (RTController *)0 )
    {
	rts = myOwner->defaultController( *this, index );

	if( rts == (RTController *)0 )
	    rts = controller;
    }

    if( actorClass         == (const RTActorClass *)0 ||
	actorClass->create == (RTActorCreator)0 )
    {
	actorClass = myOwner->defaultClass( *this, index );

	if( actorClass         == (const RTActorClass *)0 ||
	    actorClass->create == (RTActorCreator)0 )
	{
	    controller->setError( RTController::badClass,
				  RTController::FrameIncarnate, this, &index );
	    unreserve( index );
	    return (RTActor *)0;
	}
    }

    error = controller->canPlugIn( actorClass, _info );

    if( error != RTController::ok )
    {
	controller->setError( error, RTController::FrameIncarnate, this,
								   &index );

#if RTTYPECHECK_PROTOCOL == RTTYPECHECK_WARN
	if( error != RTController::protocolType )
#endif
	{
	    unreserve( index );
	    return (RTActor *)0;
	}
    }

    RTActor * actor = (*actorClass->create)( rts, this );

    if( actor == (RTActor *)0 )
    {
	controller->setError( RTController::badClass,
			      RTController::FrameIncarnate, this, &index );
	unreserve( index );
	return (RTActor *)0;
    }

    actor->_preinitialize( index, _info );

#if USE_THREADS && RTFRAME_THREAD_SAFE
    lock();
#endif

    add( actor, index );

    if( bind_ports( actor, index, (RTActorRef *)0 ) == 0 )
    {
#if USE_THREADS && RTFRAME_THREAD_SAFE
	unlock();
#endif
	controller->setError( RTController::badClass,
			      RTController::FrameIncarnate, this, &index );
	remove( index );
	delete actor;
	return (RTActor *)0;
    }

#if USE_THREADS && RTFRAME_THREAD_SAFE
    unlock();
#endif

    actor->_incarnateAllFixed();

#if OBSERVABLE
    controller->debugger()->actorIncarnated( actor );
#endif

    return actor;
}
