/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActor.h>
#include <RTController.h>
#include <RTDebugger.h>
#include <RTDiag.h>
#include <RTMessage.h>
#include <RTQFilter.h>
#include <RTSpecialSignals.h>

class RTController_destroy_filter : public RTQFilter
{
public:
    RTController_destroy_filter( RTActor      * _actor,
				 RTController * _rts )
	    : RTQFilter()
	    , actor( _actor )
	    , rts  ( _rts   )
	{
	}
    virtual ~RTController_destroy_filter( void );

    virtual int  select( RTMessage & );
    virtual void accept( RTMessage & );

private:
    RTActor      * actor;
    RTController * rts;

    // unavailable methods
    RTController_destroy_filter( const RTController_destroy_filter & );
    RTController_destroy_filter & operator=( const RTController_destroy_filter & );
}; //lint !e1712

int RTController_destroy_filter::select( RTMessage & msg )
{
    return actor == msg.receiver();
}

void RTController_destroy_filter::accept( RTMessage & msg )
{
    rts->freeMsg( &msg );
}

RTController_destroy_filter::~RTController_destroy_filter( void )
{
    actor = (RTActor      *)0;
    rts   = (RTController *)0;
}

void RTController::destroy( RTActor * actor, RTActor * owner )
{
    (void)owner; // it is used (by the threaded variant)

#if USE_THREADS
    RTController * target = actor->context();

    if( target != this )
    {
	if( send( actor, RTDestroySignal,
			 (void *)&owner, (const RTObject_class *)0,
			 Synchronous ) != 0 )
	{
	    while( owner != (RTActor *)0 )
		awaitResponse();
	}
	return;
    }
#endif

#if OBSERVABLE
    debugger()->actorDestroyed( actor );
#endif

    actor->_predestroy();

    RTController_destroy_filter filter( actor, this );

    filterMessages( actor, filter );

    delete actor;
}

//lint -save -e1738
RTController_destroy_filter::
RTController_destroy_filter( const RTController_destroy_filter & )
    : RTQFilter()
    , actor( (RTActor      *)0 )
    , rts  ( (RTController *)0 )
{
    RTDiag::panic( "Unavailable method called:\n"
"    'RTController_destroy_filter( const RTController_destroy_filter & )'" );
}
//lint -restore

RTController_destroy_filter &
RTController_destroy_filter::operator=( const RTController_destroy_filter & )
{
    RTDiag::panic( "Unavailable method called:\n"
"    'RTController_destroy_filter::operator=( const RTController_destroy_filter & )'" );

    //lint -unreachable
    return *this;
} //lint !e1529
