/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDaemon.h>
#include <RTDaemonInfo.h>
#include <RTDebugger.h>
#include <RTElasticArray.h>

void RTDaemon::refresh( int observerId, int daemonId )
{
    debugger->evtRtsStatus( observerId );

    int thisDepth = 0;

    // the actors should be refreshed in depth-order, otherwise the
    // Toolset might receive an instance before its reference.
    for( ;; )
    {
	int nextDepth = -1;

	for( int iActor = 0; iActor < actorInfo.size(); ++iActor )
	{
	    RTDaemonInfo * daemonInfo = (RTDaemonInfo *)actorInfo[ iActor ];

	    if( daemonInfo == (RTDaemonInfo *)0 )
		continue;

	    int depth = daemonInfo->actorDepth;

	    if( depth < thisDepth )
		continue;
	    else if( depth > thisDepth )
	    {
		if( nextDepth == -1 || nextDepth > depth )
		    nextDepth = depth;
	    }
	    else // depth match
	    {
		debugger->getPresence( daemonInfo->probeNr,
				       observerId,
				       daemonId );

		if( daemonInfo->daemonKind == RTDaemonInfo::InfoActor )
		    debugger->getReference( daemonInfo->probeNr, observerId );

		if( daemonInfo->daemonKind == RTDaemonInfo::InfoState )
		    debugger->getHistory( daemonInfo->probeNr,
					  daemonInfo->stateTarget );
	    }
	}

	if( nextDepth > -1 )
	    thisDepth = nextDepth;
	else
	    break;
    }
}
