/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDaemon.h>
#include <RTEventInfo.h>
#include <RTMemoryUtil.h>

static int getSettingsIndex( const char * varName )
{
    static const char * const varNames[ RTDaemon::SettLastEntry + 2 ] =
    {
	"observePresence",  // See RTEventInfo.h for order
	"observeReference",
	"observeChain",
	"observeState",
	"observeFSM",
	"observeLateFSM",
	"observeMsgIn",
	"observeMsgOut",
	"observeMsgOutIn",
	"observeTimer",
	"observeVariable",
	"observeHistory",
	"observeControl",
	"haltEnabled",      // See RTDaemon.h for order
	"haltSystem",
	"report",
	"threshold",
	"type"              // RTDaemon::SettLastEntry + 1, used for "type="
    };

    for( int iName = 0; iName < RTDaemon::SettLastEntry + 2; ++iName )
	if( RTMemoryUtil::strcmp( varNames[ iName ], varName ) == 0 )
	    return iName;

    return -1;
}

static void setCommand( int settings[] ) // Command (Run/Stop/Step) Probe
{
    settings[ RTEventInfo::EventControl ] = 1;
    settings[ RTDaemon::SettDoReport    ] = 1;
}

static void setFSM( int settings[] )     // Behavior Monitor Animation
{
    settings[ RTEventInfo::EventPresence ] = 1; // see actor presence
    settings[ RTEventInfo::EventFSM      ] = 1; // see state changes
    settings[ RTEventInfo::EventHistory  ] = 1; // see state history
    settings[ RTDaemon::SettDoReport     ] = 1;
}

static void setFrame( int settings[] )   // Structure Monitor & RTS Panel
{
    settings[ RTEventInfo::EventPresence  ] = 1; // see actor presence
    settings[ RTEventInfo::EventReference ] = 1; // see actor refs
    settings[ RTDaemon::SettDoReport      ] = 1;
}

static void setState( int settings[] )   // State Probe
{
    settings[ RTEventInfo::EventState    ] = 1;
    settings[ RTDaemon::SettRecThreshold ] = 10;
}

static void setChain( int settings[] )   // Transition Probe
{
    settings[ RTEventInfo::EventChain    ] = 1;
    settings[ RTDaemon::SettRecThreshold ] = 10;
}

static void setPort( int settings[] )    // Port Probe
{
    settings[ RTEventInfo::EventMsgIn    ] = 1; // see msgs received
    settings[ RTEventInfo::EventMsgOut   ] = 1; // see msgs sent
    settings[ RTDaemon::SettRecThreshold ] = 10;
}

static void setMSC( int settings[] )     // MSC
{
    settings[ RTEventInfo::EventPresence ] = 1; // see actor presence
    settings[ RTEventInfo::EventLateFSM  ] = 1; // see state changes later
    settings[ RTEventInfo::EventMsgOutIn ] = 1; // see messages between
    settings[ RTEventInfo::EventTimer    ] = 1; // see timeouts
    settings[ RTEventInfo::EventHistory  ] = 1; // see state history
    settings[ RTDaemon::SettDoReport     ] = 1;
}

static void setVariable( int settings[] )
{
    settings[ RTEventInfo::EventPresence ] = 1; // see actor presence
    settings[ RTEventInfo::EventVariable ] = 1;
    settings[ RTDaemon::SettDoReport     ] = 1;
}

typedef void ( * setFunc )( int [] );

struct typeLookupTable
{
    const char * type;
    setFunc      setFuncPtr;
};

static void updateType( const char * typeName, int settings[] )
{
    static const typeLookupTable typeTable[] =
    {
	{ "Command",       setCommand  },
	{ "FSM",           setFSM      },
	{ "Frame",         setFrame    },
	{ "State",         setState    },
	{ "Chain",         setChain    },
	{ "Port",          setPort     },
	{ "MSC",           setMSC      },
	{ "Variable",      setVariable },
	{ (const char *)0, (setFunc)0  }
    };

    for( const typeLookupTable * cursor = typeTable;
				 cursor->type != (const char *)0;
				 ++cursor )
    {
	if( RTMemoryUtil::strcmp( cursor->type, typeName ) == 0 )
	{
	    (*cursor->setFuncPtr)( settings );
	    break;
	}
    }
}

static int evalToInt( const char * expr )
{
    return RTMemoryUtil::strcmp( "True", expr ) == 0 ||
	   RTMemoryUtil::strcmp( "1",    expr ) == 0 ||
	   RTMemoryUtil::strcmp( "Yes",  expr ) == 0;
}

void RTDaemon::updateSettings( const char * settingsStr )
{
    if( settingsStr == (const char *)0 )
	return;

    char * workStr  = RTMemoryUtil::strdup( settingsStr );
    char * slidePtr = workStr;
    char * varPtr   = workStr;
    char * valPtr   = (char *)0;

    for(;;)
    {
	if( valPtr == (char *)0 && *slidePtr == '=' )
	{
	    *slidePtr = '\0';
	    valPtr    = ++slidePtr;
	}

	char saveChar = *slidePtr;

	if( valPtr != (char *)0 && ( saveChar == ';' || saveChar == '\0' ) )
	{
	    *slidePtr = '\0';
	    int settingsIndex = getSettingsIndex( varPtr );

	    if( settingsIndex > SettLastEntry )
		updateType( valPtr, settings );
	    else if( settingsIndex >= 0 )
		settings[ settingsIndex ] = evalToInt( valPtr );

	    varPtr = slidePtr + 1;
	    valPtr = (char *)0;
	}

	if( saveChar == '\0' )
	    break;

	++slidePtr;
    }

    delete [] workStr;
}
