/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActor.h>
#include <RTActorProbe.h>
#include <RTActorRef.h>
#include <RTActorRefProbe.h>
#include <RTActor_class.h>
#include <RTComponentDescriptor.h>
#include <RTDebugger.h>
#if USE_THREADS
#include <RTMutex.h>
#endif

void RTDebugger::probeAttention( const RTActor * target )
{
#if USE_THREADS
    _mutex->enter();
#endif

    RTActorProbe * probe = target->probe;

    if( probe != (RTActorProbe *)0 )
    {
	probe->accessFieldValue();

	if( probe->_destroyMe != 0 )
	{
	    (void)probe->disconnectProbe( (RTActor *)0 );
	    delete probe;
	}
    }

    const RTActor_class         * info = target->getActorData();
    const RTComponentDescriptor * desc = info->components;

    for( int i = info->numComponents; --i >= 0; ++desc )
    {
	const RTActorRef * actorRef = RTConstObjectOf( RTActorRef, desc->offset,
								   target );
	RTActorRefProbe  * refProbe = actorRef->probe;

	if( refProbe != (RTActorRefProbe *)0 && refProbe->_destroyMe != 0 )
	{
	    (void)refProbe->disconnectProbe( (RTActor *)0 );
	    delete refProbe;
	}
    }

#if USE_THREADS
    _mutex->leave();
#endif
}
