/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActor.h>
#include <RTController.h>
#include <RTDebugger.h>
#if USE_THREADS
#include <RTMutex.h>
#endif

void RTDebugger::step( unsigned count )
{
#if USE_THREADS
    _mutex->enter();
#endif

    mode       = Stepping;
    step_count = count;

#if OBSERVABLE
    int active_count = 0;

    if( step_count > 0U )
	evtRtsStatus();
#endif

    RTController * task = tasks;

    if( task != (RTController *)0 )
    {
	do
	{
	    switch( task->_state )
	    {
	    case RTController::waitBegin:
	    case RTController::waitContinue:
		if( step_count == 0U )
		{
		    // this is really a 'stop' command
		}
		else
		{
		    --step_count;
		    task->_state = RTController::waitDone;
#if USE_THREADS
		    wakeup( task );
#endif
#if OBSERVABLE
		    ++active_count;
#endif
		}
		break;

	    case RTController::waitDone:
	    case RTController::execUser:
#if OBSERVABLE
		++active_count;
#endif
		break;

#ifdef _lint
	    case RTController::detached:
	    case RTController::execSystem:
#endif
	    default:
		break;
	    }
	}
	while( (task = task->_next) != tasks );
    }

#if OBSERVABLE
    if( step_count == 0U && active_count == 0 )
    {
	mode = Halted;
	evtRtsStatus();
    }
#endif

#if USE_THREADS
    _mutex->leave();
#endif
}
