/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTController.h>
#include <RTDebugger.h>
#include <RTDiag.h>
#if USE_THREADS
#include <RTMutex.h>
#endif

int RTDebugger::step_begin( RTController * task )
{
    int can_go = 1;

#if USE_THREADS
    _mutex->enter();
#endif

    switch( task->_state )
    {
    case RTController::detached:
    case RTController::execUser:
	break;

    case RTController::waitBegin:
    case RTController::waitContinue:
	can_go = 0;
	break;

    case RTController::waitDone:
	task->_state = RTController::execUser;
	break;

    case RTController::execSystem:
	task->_state = RTController::waitBegin;
	switch( mode )
	{
	case Running:
	    task->_state = RTController::execUser;
	    break;

	case Stepping:
	    if( step_count == 0U )
	    {
		mode = Halted;
#if OBSERVABLE
		evtRtsStatus();
#endif
	    }
	    else
	    {
		--step_count;
		task->_state = RTController::execUser;
		break;
	    }
	    can_go = 0;
	    break;

	case Halted:
	    can_go = 0;
	    break;

	default:
	    break;
	}
	break;

    default:
	RTDiag::panic( "unexpected controller state" );
	//lint -unreachable
	break;
    }

#if USE_THREADS
    _mutex->leave();
#else
    if( can_go == 0 )
	debugIt();
#endif

    return can_go;
}
