/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDebuggerInput.h>
#include <RTDiag.h>
#include <RTDiagStream.h>
#include <RTMemoryUtil.h>

int RTDebuggerInput::getCommand( const CmdDesc * table,
				 int             none,
				 int             invalid )
{
    char buffer[ 80 ];

    if( getString( buffer, (int)sizeof( buffer ) ) == 0 )
	return none;

    char * cursor;
    char   ch;

    for( cursor = buffer; (ch = *cursor) != '\0'; ++cursor )
	if( 'A' <= ch && ch <= 'Z' )
	    *cursor = (char)( ch - 'A' + 'a' );

    bool ambiguous = false;
    int  len       = cursor - buffer; //lint !e946 !e947

    for( const CmdDesc * cmd = table; cmd->minAbbr != 0; ++cmd )
    {
	if( RTMemoryUtil::strcmp( buffer, cmd->name, len ) != 0 )
	    continue;
	if( len >= cmd->minAbbr )
	    return cmd->cmd;
	if( ! ambiguous )
	{
	    RTDiag::out()
		<< "Command '" << buffer << "' is ambiguous; choices are:\n";
	    ambiguous = true;
	}
	RTDiag::out() << "  " << cmd->name;
    }

    if( ambiguous )
    {
	RTDiag::out() << "\n";
	return none;
    }

    return invalid;
}
