/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */
#include <RTActorId.h>
#include <RTComponentDescriptor.h>
#include <RTDebugger.h>
#include <RTDiag.h>
#include <RTDiagStream.h>
#include <RTLocalConnector.h>
#include <RTMain.h>
#include <RTSpecialSignals.h>
#include <RTSuperActor.h>
#include <RTSystemDescriptor.h>
#include <RTVersion.h>

#if USE_THREADS
#   include <RTThread.h>
#   include <RTPeerController.h>
#else
#   include <RTSoleController.h>
#endif

static const char config_info[] =
{
#if RTS_COUNT
    ',', '+', 'C',
#endif
#if ! OBJECT_DECODE
    ',', '-', 'D',
#endif
#if   OTRTSDEBUG == DEBUG_NONE
    ',', 'G', '0',
#elif OTRTSDEBUG == DEBUG_TERSE
    ',', 'G', '1',
#endif
#if ! OBJECT_ENCODE
    ',', '-', 'E',
#endif
#if   RTFRAME_CHECKING == RTFRAME_CHECK_NONE
    ',', 'F', '0',
#elif RTFRAME_CHECKING == RTFRAME_CHECK_LOOSE
    ',', 'F', '1',
#endif
#if ! RTFRAME_THREAD_SAFE
    ',', '-', 'F', 's',
#endif
#if ! HAVE_INET
    ',', '-', 'I',
#endif
#if ! LOG_MESSAGE
    ',', '-', 'L',
#endif
#if ! OBSERVABLE
    ',', '-', 'O',
#endif
#if PURIFY
    ',', '+', 'P',
#endif
#if   RTTYPECHECK_PROTOCOL == RTTYPECHECK_DONT
    ',', '-', 'T', 'p',
#elif RTTYPECHECK_PROTOCOL == RTTYPECHECK_FAIL
    ',', '+', 'T', 'p',
#endif
#if   RTTYPECHECK_RECEIVE == RTTYPECHECK_DONT
    ',', '-', 'T', 'r',
#elif RTTYPECHECK_RECEIVE == RTTYPECHECK_FAIL
    ',', '+', 'T', 'r',
#endif
#if   RTTYPECHECK_SEND == RTTYPECHECK_DONT
    ',', '-', 'T', 's',
#elif RTTYPECHECK_SEND == RTTYPECHECK_FAIL
    ',', '+', 'T', 's',
#endif
#if RTUseBitFields
    ',', '+', 'b',
#endif
#if RTS_COMPATIBLE < RT_VERSION_NUMBER
    ',', '+', 'c',
#endif
#if DEFER_IN_ACTOR
    ',', '+', 'd',
#endif
#if ! RTS_INLINES
    ',', '-', 'i',
#endif
#if ! RTMESSAGE_PAYLOAD_SIZE
    ',', '-', 'm',
#endif
#if ! INTEGER_POSTFIX
    ',', '-', 'p',
#endif
#if ! RTREAL_INCLUDED
    ',', '-', 'r',
#endif
#if SUSPEND
    ',', '+', 's',
#endif
#if ! USE_THREADS
    ',', '-', 't',
#endif
#if   RTStateId_MaxSize == 1
    ',', '-', 'Z',
#elif RTStateId_MaxSize == 4
    ',', '+', 'Z',
#endif
    ')', '\0'
};

static void showBanner( void )
{
    RTDiagStream & out = RTDiag::out();

    out.lock()
	<< "Rational Rose RealTime C++ Target Run Time System\n"
	<< RT_VERSION_STRING;

    if( config_info[ 1 ] != '\0' )        // more than ")"?
	out << " (" << &config_info[ 1 ]; // skip leading comma

    out << "\nCopyright (c) 1999-" RT_VERSION_YEAR " Rational Software\n";
    out.flush();
    out.unlock();
}

// The only fields that matter here are numInterfaces and interfaces.

static const RTComponentDescriptor super_info =
{
    (const char *)0,                    // name
    (const RTActorClass *)0,            // defaultClass
    (RTFieldOffset)0,                   // offset
    0,                                  // id
    RTComponentDescriptor::Optional,    // kind
    0,                                  // substitutable
    1,                                  // replication

    0, (const RTInterfaceDescriptor *)0,
    0, (const RTBindingDescriptor   *)0
};

int RTMain::mainLine( void )
{
    targetStartup();

    showBanner();

    RTLocalConnector localConnector;

    if( localConnector.install( (RTController *)0, "" ) == 0 )
	RTDiag::panic( "Can't install root layer connector" );

    installHandlers();

#if OBJECT_DECODE
    RTTypeInstaller::installAll();
#endif

#if OTRTSDEBUG != DEBUG_NONE
    RTDebugger         debugger_;

    RTDebugger * const debugger = &debugger_;
#else
    RTDebugger * const debugger = (RTDebugger *)0;
#endif

    //===============================//
    // create all system controllers //
    //===============================//

#if USE_THREADS
    _controller = new RTPeerController( debugger, "main" );
#else
    _controller = new RTSoleController( debugger, "main" );
#endif

#if USE_THREADS && OTRTSDEBUG != DEBUG_NONE
    // create debugger thread
    RTThread * debugThread = new RTThread( debugger, debugStackSize,
						     debugPriority );
#endif // USE_THREADS && OTRTSDEBUG != DEBUG_NONE

    //============================================//
    // create application controllers and threads //
    //============================================//

#if USE_THREADS
    _system->createThreads( debugger );
#endif
    _system->logicalThreads( _controller );

    //======================//
    // create 'super' actor //
    //======================//

    _superActor = new RTSuperActor( _controller );
    _superActor->_preinitialize( 0, &super_info );

    if( _controller->send( _superActor, RTInitSignal,
			   (const void *)0, (const RTObject_class *)0,
			   Synchronous ) == 0 )
	RTDiag::panic( "Can't initialize super actor" );

    RTController * mainController = _controller;

    mainController->mainLoop();

    _controller = (RTController *)0;

#if USE_THREADS
    _system->deleteThreads();

#if OTRTSDEBUG != DEBUG_NONE
    delete debugThread;
#endif
#endif // USE_THREADS

    delete mainController;

    if( localConnector.uninstall( (RTController *)0, "" ) == 0 )
	RTDiag::panic( "Can't uninstall root layer connector" );

    RTLayerConnector::cleanup();

    targetShutdown();

    return 0;
}
