/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTController.h>
#include <RTMessage.h>
#include <RTSpecialSignals.h>
#include <RTTimerActor.h>
#include <RTTimerId.h>
#include <RTTimerNode.h>

void RTTimerNode::_cancel( RTController * controller )
{
    RTMessage * message = msg;

    if( message != (RTMessage *)0 )
    {
	msg = (RTMessage *)0;

	if( message->signal == RTTimeoutSignal )
	    // we must wait for the message to be delivered to finish up
	    message = (RTMessage *)0;

	RTTimerActor * timerActor = RTTimerActor::soleInstance();

	if( timerActor != (RTTimerActor *)0 )
	    timerActor->cancel( this );
    }

    // invalidate all RTTimerId objects that refer to this node
    _disconnectAll();

    RTController * target = owner;

    if( active == 0 )
	target->freeTimerNode( this );

    if( message != (RTMessage *)0 )
    {
#if RTS_COUNT && USE_THREADS
	if( controller != target )
	{
	    message->clear();
	    message->setDestination( (RTActor *)0, (RTProtocol *)0, 0 );
	    message->setPriority( Synchronous );
	    message->setSignal( RTWakeupSignal );
	    target->peer_receive( message );
	}
	else
#endif
	{
	    controller->freeMsg( message );
	}
    }
}

int RTTimerNode::cancel( RTController * controller, RTTimerId & id )
{
    if( id.node == (RTTimerNode *)0 )
	return 0;

#if USE_THREADS
    RTTimerNode::critical.enter();
#endif

    RTTimerNode * node = id.node;

    if( node == (RTTimerNode *)0 )
    {
#if USE_THREADS
	RTTimerNode::critical.leave();
#endif
	return 0;
    }

    node->_cancel( controller );

#if USE_THREADS
	RTTimerNode::critical.leave();
#endif

    return 1;
}
