/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTActorSlot_h__
#define __RTActorSlot_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTConfig_h__
#include <RTConfig.h>
#endif

class RTActor;

// If the replication factor, n, of an optional reference satisfies
// ( 2 <= n < threshold ), we allocate n + 1 RTSmallOptional objects.
// The first one serves as the anchor for the free list.
// This depends on two things:
// 1.  int is no larger than a pointer, and
// 2.  all addresses allocated on the heap are even.

struct RTSmallOptional
{
    RTS_INLINE static int supports( int size );
    RTS_INLINE void       insert( int prev, int next );
    RTS_INLINE int        inuse( void ) const;
    RTS_INLINE void       prev( int );
    RTS_INLINE int        prev( void ) const;
    RTS_INLINE void       next( int );
    RTS_INLINE int        next( void ) const;

    RTActor * _actor;

    enum
    {
	// we assume sizeof( int ) <= sizeof( RTActor * )
	link_bits  = 4 * (int)sizeof( int ) - 1
      , threshold  = 1 << link_bits
      , link_mask  = threshold - 1
      , next_shift = 1
      , prev_shift = next_shift + link_bits
    };
};

// If ( n >= threshold ), we allocate n + 1 RTLargeOptional objects.
// The first one serves as the anchor for the free list.
// The next pointer is nil when the entry has been reserved.

struct RTLargeOptional
{
    RTLargeOptional     * next;
    union
    {
	RTLargeOptional * prev;
	RTActor         * actor;
    };
};

#if RTS_INLINES
#include <RTActorSlot.inl>
#endif

#endif // __RTActorSlot_h__
