/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTToolsetObserver_h__
#define __RTToolsetObserver_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTObserver_h__
#include <RTObserver.h>
#endif

#ifndef __endeBoth_h__
#include <endeBoth.h>
#endif

#ifndef __endeTarg_h__
#include <endeTarg.h>
#endif

class RTToolsetObserver : public RTObserver
{
private:
    TO_bufferInfo rxBuffer;
    TO_bufferInfo txBuffer;
    TO_bufferInfo txTempBuffer;
    TO_structInfo structInfo;

public:
		  RTToolsetObserver( RTDebugger  * theDebugger,
				     RTTcpSocket * theSocket,
				     int           theObsId,
				     const char  * initBuf = (const char *)0,
				     int           initLen = 0 );
    virtual       ~RTToolsetObserver( void );

    virtual int   evtChain(        int                  daemonId,
				   const RTDaemonInfo & daemonInfo,
				   const RTEventInfo  & eventInfo );

    virtual int   evtPresence(     int                  daemonId,
				   const RTDaemonInfo & daemonInfo,
				   const RTEventInfo  & eventInfo );

    virtual int   evtReference(    int                  daemonId,
				   const RTDaemonInfo & daemonInfo,
				   const RTEventInfo  & eventInfo );

    virtual int   evtMessageIn(    int                  daemonId,
				   const RTDaemonInfo & daemonInfo,
				   const RTEventInfo  & eventInfo );

    virtual int   evtMessageOut(   int                  daemonId,
				   const RTDaemonInfo & daemonInfo,
				   const RTEventInfo  & eventInfo );

    virtual int   evtMessageOutIn( int                  daemonId,
				   const RTDaemonInfo & sndDaemonInfo,
				   const RTDaemonInfo & recDaemonInfo,
				   const RTEventInfo  & eventInfo );

    virtual int   evtRtsStatus(    int                  daemonId,
				   const RTEventInfo  & eventInfo );

    virtual int   evtVariable(     int                  daemonId,
				   const RTDaemonInfo & daemonInfo,
				   const RTEventInfo  & eventInfo );

    virtual int   evtHistory(      int                  daemonId,
				   const RTDaemonInfo & daemonInfo,
				   const RTEventInfo  & eventInfo );

    virtual int   receiveData ( void );
    virtual int   transmitData( void );

private:
    void          encoDeco( char * buffer, int bufLen );

    void          handleDaemon( void );
    void          handleActor( void );
    void          handlePort( void );
    void          handleState( void );
    void          handleChain( void );
    void          handleVariable( void );
    void          accessVariable( void );
    void          handleInject( void );

    void          sendResultCode( int resultCode );

    void          sendTxBuffer( void );

    static char * buildString( const char * actor_path,
			       int          actor_index,
			       const char * component_name = 0 );

    // The data in message events is limited to this many bytes.
    enum { msgDataBufSize = 1000 };

private:
    // unavailable methods
    RTToolsetObserver( const RTToolsetObserver & );
    RTToolsetObserver & operator=( const RTToolsetObserver & );
}; //lint !e1712

#endif // __RTToolsetObserver_h__
