/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifdef PRAGMA
#pragma implementation "RTThreadInfo.h"
#endif

#include <RTDiag.h>
#include <RTJob.h>
#include <RTMemoryUtil.h>
#include <RTThread.h>
#include <RTThreadInfo.h>

#include <chorus.h>

static void set_name( const char * name )
{
    // put our name on the thread
    char short_name[ K_THREADNAMEMAX ];

    RTMemoryUtil::strcpy( short_name, name, sizeof( short_name ) - 1 );

    if( threadName( K_MYACTOR, K_MYSELF, (char *)0, short_name ) )
	RTDiag::panic( "threadName" );
}

static void thread_run( void )
{
    // get the thread info pointer from our soft register
    RTThreadInfo * info = (RTThreadInfo *)threadLoadR();

    set_name( info->job->name() );

    info->job->mainLoop();

    info->done = 1;
    info->sync.signal();

    for(;;)
	if( threadSuspend( K_MYACTOR, K_MYSELF ) != K_OK )
	    RTDiag::panic( "threadSuspend" );
}

RTThread::RTThread( RTJob * job, int stack_size, int prio )
    : _job (                   job   )
    , _info( new RTThreadInfo( job ) )
{
    KnActorPrivilege     actorP;
    KnThreadDefaultSched schedparam;
    KnDefaultStartInfo_f startInfo;
    KnThreadSoftCtx      newcontext;
    KnThreadSoftCtx      oldcontext;

    // set defaults startInfo fields
    startInfo.dsType            = K_DEFAULT_START_INFO;
    startInfo.dsSystemStackSize = K_DEFAULT_STACK_SIZE;

    // set the entry point for the new thread
    startInfo.dsEntry = (KnPc)thread_run;

    // get the actor's privelege
    if( actorPrivilege( K_MYACTOR, &actorP, 0 ) != K_OK )
	RTDiag::panic( "actorPrivilege" );

    // set the priority
    schedparam.tdClass    = K_SCHED_DEFAULT;
    schedparam.tdPriority = prio;

    if( actorP == K_SUPACTOR )
    {
	startInfo.dsPrivilege        =  K_SUPTHREAD;
	startInfo.dsUserStackPointer = (void *)0;
	newcontext.supReg            = (long)_info;
    }
    else
    {
	if( stack_size == 0 )
	    RTDiag::panic( "zero stack size" );

	char * userStack = new char [ stack_size ];

	if( userStack == (char *)0 )
	    RTDiag::panic( "can't allocate stack" );

	startInfo.dsPrivilege        = K_USERTHREAD;
	startInfo.dsUserStackPointer = userStack + stack_size - sizeof( long );
	newcontext.userReg           = (long)_info;
    }

    // create the thread in the active state
    if( threadCreate( K_MYACTOR, &_info->tid,
		      K_SUSPENDED, &schedparam, &startInfo ) != K_OK )
	RTDiag::panic( "threadCreate" );

    if( threadContext( K_MYACTOR, _info->tid,
		       K_SOFTCTX, &oldcontext, &newcontext ) != K_OK )
	RTDiag::panic( "threadContext" );

    // make it go
    if( threadResume( K_MYACTOR, _info->tid ) )
	RTDiag::panic( "threadActivate" );
}

RTThreadInfo::RTThreadInfo( RTJob * _job )
    : sync(      )
    , job ( _job )
    , tid (   -1 )
    , done(    0 )
{
}

RTThreadInfo::~RTThreadInfo( void )
{
    if( tid != -1 )
    {
	while( done == 0 )
	    sync.wait();

	if( threadDelete( K_MYACTOR, tid ) != K_OK )
	    RTDiag::panic( "threadDelete" );
    }

#ifdef _lint
    job = (RTJob *)0;
#endif
}
