/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifdef PRAGMA
#pragma implementation "RTThreadInfo.h"
#endif

#include <RTDiag.h>
#include <RTJob.h>
#include <RTThread.h>
#include <RTThreadInfo.h>

static void * thread_run( void * arg )
{
    ((RTJob *)arg)->mainLoop();
}

RTThread::RTThread( RTJob * job, int stack_size, int prio )
    : _job ( job )
    , _info( new RTThreadInfo )
{
    pthread_attr_t attr;

#if TARGET_LYNX >= 31
    if( pthread_attr_init( &attr ) != 0 )
	RTDiag::panic( "pthread_attr_init" );
#else
    if( pthread_attr_create( &attr ) != 0 )
	RTDiag::panic( "pthread_attr_create" );
#endif

    pthread_attr_setstacksize( &attr, stack_size );
    pthread_attr_setinheritsched( &attr, PTHREAD_INHERIT_SCHED );

#if TARGET_LYNX >= 31
    if( pthread_create( &_info->tid, &attr, thread_run, job ) != 0 )
#else
    if( pthread_create( &_info->tid, attr, thread_run, job ) != 0 )
#endif
	RTDiag::panic( "pthread_create" );
    else
    {
	_info->valid = 1;

#if TARGET_LYNX >= 31
	int                policy;
	struct sched_param param;

	if( pthread_getschedparam( _info->tid, &policy, &param ) != 0 )
	    RTDiag::panic( "pthread_getschedparam" );
	param.sched_priority = prio;
	if( pthread_setschedparam( _info->tid, policy, &param ) != 0 )
	    RTDiag::panic( "pthread_setschedparam" );
#else
	pthread_setprio( _info->tid, prio );
#endif
    }
}

RTThreadInfo::RTThreadInfo( void )
    : valid( 0 )
{
}

RTThreadInfo::~RTThreadInfo( void )
{
    if( valid == 0 )
	return; // never successfully created

    void * status = (void *)0;

    if( pthread_join( tid, &status ) != 0 )
	RTDiag::panic( "pthread_join" );
}
