/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTIOMonitor.h>
#include <RTMemoryUtil.h>
#include <RTNucleus.h>
#include <RTTimespec.h>
#include <RTtcp.h>

int RTIOMonitor::wait( const RTTimespec * timeout )
{
    fd_set rSet;
    fd_set wSet;
    fd_set eSet;

    RTMemoryUtil::memset( &rSet, 0, (int)sizeof( rSet ) );
    RTMemoryUtil::memset( &wSet, 0, (int)sizeof( wSet ) );
    RTMemoryUtil::memset( &eSet, 0, (int)sizeof( eSet ) );

    int        max_fd = -1;
    Entry    * entry;
    unsigned   index;

    for( entry = _entries, index = 0U; index < _used; ++entry, ++index )
    {
	unsigned fd   = entry->fd;
	int      bits = entry->bits &= RequestMask;

	if( max_fd < (int)fd )
	    max_fd = (int)fd;

	//lint -save -e647 -e661 -e662 -e912
	if( ( bits & CanRead   ) != 0 ) NU_FD_Set( fd, &rSet );
	if( ( bits & CanWrite  ) != 0 ) NU_FD_Set( fd, &wSet );
	if( ( bits & Exception ) != 0 ) NU_FD_Set( fd, &eSet );
	//lint -restore
    }

    UNSIGNED suspend;
    long     ticks;

    if( timeout == (const RTTimespec *)0 )
	suspend = NU_SUSPEND;
    else if( (ticks = timeout->ticksFromNow( RTNucleus::clock_period )) <= 0 )
	suspend = NU_NO_SUSPEND;
    else
	suspend = (UNSIGNED)ticks;

    if( NU_Select( max_fd + 1, &rSet, &wSet, &eSet, suspend ) != NU_SUCCESS )
	return 0;

    for( entry = _entries, index = 0U; index < _used; ++entry, ++index )
    {
	unsigned fd   = entry->fd;
	int      bits = 0;

	//lint -save -e647 -e912
	if( NU_FD_Check( fd, &rSet ) ) bits |= ResultCanRead;
	if( NU_FD_Check( fd, &wSet ) ) bits |= ResultCanWrite;
	if( NU_FD_Check( fd, &eSet ) ) bits |= ResultException;
	//lint -restore

	entry->bits |= bits;
    }

    return 1;
}
