/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDiag.h>
#include <RTMain.h>
#include <RTOse.h>

#include <dbgprintf.h>
#include <bios.h>

extern "C" void init_dbgread( void );

// include error handler here
extern "C" OSBOOLEAN TargetRTS_err_hnd( OSBOOLEAN user_called,
					OSERRCODE ecode,
					OSERRCODE extra )
{
    OSERRCODE errcode     = ecode & OSE_EERROR_MASK;
    OSERRCODE subcode     = ecode & OSE_ESUBCODE_MASK;
    PROCESS   sysErrorPid = current_process();

    // check for inet error
    if( INET_ERROR( subcode ) )
    {
	OSERRCODE origin = INET_EFUNCTION( subcode );
	OSERRCODE cause  = INET_ECAUSE( subcode );

	dbgprintf( " INET error : origin 0x%08x cause 0x%08x\n",
		   origin, cause );
    }
    else
    {
	dbgprintf( " sub 0x%04x, error 0x%04x\n", subcode >> 16, errcode );
    }

    dbgprintf( "process %ld.%ld", sysErrorPid >> 16, sysErrorPid & 0xFFFF );

    dbgprintf( " %s%sextra 0x%08lx\n",
	       ( ecode & OSE_EFATAL_MASK ) != 0 ? "fatal, "       : "",
	       user_called                      ? "user called, " : "",
	       (unsigned long)extra );

    if( ecode == 0xFFFFFFFF ) /* This is an assert */
    {
	struct assert_extra
	{
	    const char * expr;
	    const char * file;
	    unsigned     line;
	};

	const assert_extra * info = (const assert_extra *)extra;

	dbgprintf( "ASSERT: expr: %s file: %s line: 0x%08lx\n",
		   info->expr, info->file, info->line );
    }

    return 1;
}

void RTMain::targetStartup( void )
{
    if( ! biosOpen( "dbgprintf" ) )
	RTDiag::panic( "dbgprintf" );

    init_dbgread();
}
