/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifdef PRAGMA
#pragma implementation "RTThreadInfo.h"
#endif

#include <RTDiag.h>
#include <RTDiagStream.h>
#include <RTElasticArray.h>
#include <RTJob.h>
#include <RTThread.h>

#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <stdlib.h>
#include <string.h>
#include <process.h>
#include <sys/sched.h>

static RTElasticArray threads;

static int thread_func( void * arg )
{
    RTJob * job = (RTJob *)arg;

    job->mainLoop();

    job->shutdown();

    return 0;
}

void RTDiag::panic( const char * message )
{
    pid_t me = getpid();

    out() << "panic: " << message << "\n";

    for( unsigned i = 0; i < threads.size(); ++i )
    {
	pid_t pid = (pid_t)threads[ i ];

	if( pid != me )
	    kill( pid, SIGKILL );
    }

    exit( 1 );
}

RTThread::RTThread( RTJob * job, unsigned stack_size, int prio )
    : _job       ( job        )
    , _stack_size( stack_size )
    , _stack     ( (char *)0  )
    , _info      ( (void *)0  )
{
    _stack = new char[ _stack_size ];

    // make sure the main "thread" is available to panic
    if( threads[ 0 ] == (void *)0 )
	threads[ 0 ] = (void *)getpid();

    pid_t pid = tfork( _stack, _stack_size, thread_func, (void *)job, 0 );

    if( pid == -1 )
	RTDiag::panic( "tfork" );

    else if( setprio( pid, prio ) == -1 )
	RTDiag::panic( "setprio" );

    else
    {
	threads[ threads.unusedIndex() ] = (void *)pid;

	// wait for the new job to be started before returning
	job->waitUntil( RTJob::started );
    }
}
