#! perl -w

#
#  Licensed Materials - Property of IBM                               
#  Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
#  Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
#  U.S. Government Users Restricted Rights -  Use, duplication or     
#  disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
# 
#
# Example usage:
# ===============
# perl link.pl ARCH=386 -o TopCapsule \
#   main.o %OLIST user_obj.o -LX -lx ...
#
# The general algorithm:
#   Translate each OLIST to OLIST.cmd
#   dld -r -o RTmunch.o main.o -T OLIST.cmd user_obj.o -LX -lx
#   nm386 RTmunch.o | munch > RTctor.c
#   dcc -o RTctor.o -c RTctor.c
#   dld -r -o TopCapsule RTctor.o RTmunch.o

# Win9x command.com does not interact with perl according to win32 standard.
# We therefore need to call some functions in a non-standard way.
#
# This detects if the system is Win95 for that purpose.

sub mySystem
{
    my( $OS, $ROSERT_HOST );

    $OS          = $ENV{'OS'}          || '?';
    $ROSERT_HOST = $ENV{'ROSERT_HOST'} || '?';

	return system( join( ' ', @_ ) );
}

sub report_ges_error
{
    my( $ERROR ) = @_;

    print "description '$ERROR' severity 'error'\n";
    print "result 'failure'\n";

    exit 1;
}

$arch_pattern = 'ARCH=(pentium|68k|960|ppc|simpc|simso)';
#$arch_pattern = 'ARCH=(pentium|ppc|mips|arm|sparc|sh)';//sairam

sub show_usage
{
    &report_ges_error( "Usage: $0 $arch_pattern options ... " );
}

&show_usage() unless $#ARGV >= 1;

if( $ARGV[0] =~ m{^$arch_pattern$} )
{
    $arch = $1;
    shift @ARGV;
}
else
{
    &show_usage();
}

&report_ges_error( "WIND_BASE is not set" )
	unless( $WIND_BASE = $ENV{'WIND_BASE'} );

# commands
$cc_cmd  = "dcc";
$ld_cmd  = "dld";

if ( $arch eq 'simpc' )
{
$munch   = "tclsh \"$WIND_BASE/host/resource/hutils/tcl/munch.tcl\""
		 . " -c pentium";
}
elsif ( $arch eq 'ppc' )
{
$munch   = "tclsh \"$WIND_BASE/host/resource/hutils/tcl/munch.tcl\""
		 . " -c $arch";
}

$nm_cmd  = "ddump -Ng";

# filenames
$prelink = 'RTmunch.o';
$ctor_c  = 'RTctor.c';
$ctor_o  = 'RTctor.o';
$output  = 'a.out';

$status   = 0;                              # assume all is well
$cmdfile  = 'ld000';                        # command filename pattern
@tmpfiles = ( $prelink, $ctor_c, $ctor_o ); # temporary files

# These are for target-specific flags.

@prelink_flags = ();
@compile_flags = ();
@final_link_flags = ();

if( $arch eq 'simpc' )
{
    push @prelink_flags, '-tX86LH:vxworks60';
    push @compile_flags, '-tX86LH:vxworks60', '-DCPU=SIMNT';
    push @final_link_flags, '-tX86LH:vxworks60';
}
else
{
    push @prelink_flags, '-tPPC603FH:vxworks60';
    push @compile_flags, '-tPPC603FH:vxworks60';
    push @final_link_flags, '-tPPC603FH:vxworks60';
}

@command  = ( $ld_cmd, @prelink_flags, '-X', '-r5', '-r4', '-o', $prelink );

@script_intro = ();

# Translate all indirect object file lists (identified by %) into linker
# command files for use in the prelink step.

foreach $arg ( @ARGV )
{
    if( $arg eq '-o' )
    {
	$lasto = 1;
    }
    elsif( $lasto )
    {
	$output = $arg;
	$lasto  = 0;
    }
    elsif( $arg =~ m{^%(.*)$} )
    {
	$in = $1;
	if( ! open( IN, "<$in" ) )
	{
	    print STDERR "link.pl: Can't read '$in': $!\n";
	    $status = 1;
	}
	else
	{
	    $out = "$cmdfile.cmd";
	    ++$cmdfile;
	    if( ! open( OUT, ">$out" ) )
	    {
		print STDERR "link.pl: Can't create '$out': $!\n";
		$status = 1;
	    }
	    else
	    {
		while( <IN> )
		{
		    chomp;
		    print OUT "$_.o\n";
		}
		close( OUT );

		push @command, @script_intro, "-@"."$out";
		@script_intro = ();
		push @tmpfiles, $out;
	    }
	    close( IN );
	}
    }
    else
    {
	push @command, $arg;
    }
}

# prelink everything
$status = &mySystem( @command ) if $status == 0;

# munch the big object file
$status = &mySystem( "$nm_cmd $prelink | $munch > $ctor_c" )
				if $status == 0;

# compile the constructor/destructor tables
if( $arch eq 'simpc' )
{
$status = &mySystem( $cc_cmd, @compile_flags, '-Xdollar-in-ident',
		     '-Xansi', '-Xlint', '-Xstrings-in-text=0', '-DCPU=SIMNT', '-DTOOL_FAMILY=diab',
		     '-DTOOL=diab', '-DUSE_THREADS', '-o', $ctor_o, '-c', $ctor_c )
				if $status == 0;
}
else
{
$status = &mySystem( $cc_cmd, @compile_flags, '-Xdollar-in-ident',
		     '-Xansi', '-Xlint', '-Xstrings-in-text=0', '-DCPU=PPC603', '-DTOOL_FAMILY=diab',
		     '-DTOOL=diab', '-DUSE_THREADS', '-o', $ctor_o, '-c', $ctor_c )
				if $status == 0;
}

# final link of the application
$status = &mySystem( $ld_cmd, @final_link_flags, '-X', '-r', '-r4', '-o', $output, $ctor_o, $prelink )
			if $status == 0;

# remove our temporary files
unlink( @tmpfiles );

exit $status if $status < 256;
exit $status / 256;
